package com.fr.design.chart;

import com.fr.base.chart.chartdata.CallbackEvent;
import com.fr.chart.chartattr.ChartCollection;
import com.fr.chartx.attr.ChartProvider;
import com.fr.design.data.DesignTableDataManager;
import com.fr.design.data.datapane.TableDataComboBox;
import com.fr.design.data.tabledata.wrapper.TableDataWrapper;
import com.fr.design.gui.ibutton.UIButton;
import com.fr.design.gui.icombocheckbox.UIComboCheckBox;
import com.fr.design.gui.ilable.UILabel;
import com.fr.design.i18n.Toolkit;
import com.fr.design.layout.FRGUIPaneFactory;
import com.fr.design.utils.gui.GUICoreUtils;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListCellRenderer;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
 * @author Bjorn
 * @version 10.0
 * Created by Bjorn on 2020-05-29
 */
public class AutoChartTypePane extends ChartWizardPane implements CallbackEvent {

    private JList chartViewList;
    private DefaultListModel chartResultModel;
    private UIButton refreshButton;

    private TableDataComboBox tableNameComboBox;
    private UIComboCheckBox dataFieldBox;

    public AutoChartTypePane() {
        this.setLayout(FRGUIPaneFactory.createBorderLayout());
        initButtonGroup();
        initRefreshLabel();
        initDataFiledBox();
        JPanel contentPane = createContentPane();

        chartViewList = new JList();

        chartResultModel = new DefaultListModel();
        chartViewList.setModel(chartResultModel);
        chartViewList.setVisibleRowCount(0);
        chartViewList.setLayoutOrientation(JList.HORIZONTAL_WRAP);
        chartViewList.setCellRenderer(iconCellRenderer);

        JScrollPane subListPane = new JScrollPane(chartViewList);

        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, contentPane, subListPane);
        splitPane.setDividerLocation(60);
        splitPane.setBorder(BorderFactory.createTitledBorder(Toolkit.i18nText("Fine-Design_Chart_M_Popup_Auto_Chart_Type")));
        this.add(splitPane);
    }

    ListCellRenderer iconCellRenderer = new DefaultListCellRenderer() {
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
            this.setText("");

            AutoChartIcon chartIcon = (AutoChartIcon) value;
            this.setIcon(chartIcon);
            setHorizontalAlignment(UILabel.CENTER);
            if (isSelected) {
                // 深蓝色.
                this.setBackground(new Color(57, 107, 181));
                this.setBorder(GUICoreUtils.createTitledBorder(chartIcon.getChartName(), Color.WHITE));
            } else {
                this.setBorder(GUICoreUtils.createTitledBorder(chartIcon.getChartName()));
            }
            return this;
        }
    };

    private JPanel createContentPane() {
        JPanel panel = FRGUIPaneFactory.createNormalFlowInnerContainer_S_Pane();

        JPanel tableDataPane = FRGUIPaneFactory.createNormalFlowInnerContainer_S_Pane();
        panel.add(tableDataPane);
        tableDataPane.add(new UILabel(Toolkit.i18nText("Fine-Design_Chart_Table_Data") + ":"));
        tableNameComboBox.setPreferredSize(new Dimension(96, 20));
        tableDataPane.add(tableNameComboBox);

        JPanel areaNamePane = FRGUIPaneFactory.createNormalFlowInnerContainer_S_Pane();
        panel.add(areaNamePane);
        areaNamePane.add(new UILabel(Toolkit.i18nText("Fine-Design_Chart_Data_Field") + ":"));
        areaNamePane.add(dataFieldBox);
        panel.add(refreshButton);
        return panel;
    }

    private void initButtonGroup() {
        dataFieldBox = new UIComboCheckBox(new Object[0]);
        dataFieldBox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                checkButtonState();
            }
        });
    }

    private void initDataFiledBox() {
        tableNameComboBox = new TableDataComboBox(DesignTableDataManager.getEditingTableDataSource());
        tableNameComboBox.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    refreshBox();
                    checkButtonState();
                }
            }
        });
    }

    private void checkButtonState() {
        if (tableNameComboBox.getSelectedItem() != null && dataFieldBox.getSelectedValues().length > 0) {
            refreshButton.setEnabled(true);
        } else {
            refreshButton.setEnabled(false);
        }
    }

    private void refreshBox() {
        TableDataWrapper dataWrap = tableNameComboBox.getSelectedItem();

        if (dataWrap == null) {
            return;
        }
        dataFieldBox.clearText();

        List<String> columnNameList = dataWrap.calculateColumnNameList();

        dataFieldBox.refreshCombo(columnNameList.toArray());
    }

    private void initRefreshLabel() {
        refreshButton = new UIButton(Toolkit.i18nText("Fine-Design_Chart_Recommend"));
        refreshButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                calculateAutoChart();
            }
        });
        refreshButton.setEnabled(false);
    }

    private void calculateAutoChart() {
        chartResultModel.clear();
    }

    @Override
    public void populate(ChartCollection cc) {

    }

    @Override
    public void update(ChartCollection cc) {
        if (chartViewList.getSelectedIndex() < 0) {
            return;
        }
        AutoChartIcon chartIcon = (AutoChartIcon) chartViewList.getSelectedValue();
        ChartProvider chartProvider = chartIcon.getChartCollection().getSelectedChartProvider(ChartProvider.class);
        if (cc.getChartCount() > 0) {
            cc.setSelectChart(chartProvider);
        } else {
            cc.addChart(chartProvider);
        }
    }

    public void populate(String tableName, String[] dataFields) {
        tableNameComboBox.setSelectedTableDataByName(tableName);
        Map<Object, Boolean> map = new HashMap();
        for (String dataField : dataFields) {
            map.put(dataField, true);
        }
        dataFieldBox.setSelectedValues(map);
    }

    @Override
    public void callback() {
        this.repaint();
    }
}
